"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.getCommonPrefix = exports.numberFormat = exports.normalizeData = exports.saveData = exports.roundDate = exports.dateParser = void 0;

var _lodash = require("lodash");

var _d = require("d3");

// TODO remove lodash
var dateParser = function dateParser(dt) {
  if (typeof dt === 'number' || typeof dt === 'string') {
    return new Date(Number(dt));
  }

  return dt;
};

exports.dateParser = dateParser;

var roundDate = function roundDate(date, timeUnit) {
  var unixDate = date / 1000;
  var x = Math.round(unixDate / timeUnit);
  return dateParser(x * timeUnit);
};

exports.roundDate = roundDate;

var saveData = function saveData() {
  var filename = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : String;
  var blob = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : Blob;
  var uri = URL.createObjectURL(blob);
  var link = document.createElement('a');

  if (typeof link.download === 'string') {
    document.body.appendChild(link); //Firefox requires the link to be in the body

    link.download = filename;
    link.href = uri;
    link.click();
    document.body.removeChild(link); //remove the link when done
  } else {
    location.replace(uri);
  }
};
/**
 * Expecting
 * @param {[name|date]: {c}} serie
 * @param {[name|date]: {[name]: {c}}} serie
 *
 */


exports.saveData = saveData;
var hourParseTime = (0, _d.utcParse)('%Y-%m-%d %H');
var dayParseTime = (0, _d.utcParse)('%Y-%m-%d');

var formatKey = function formatKey(key) {
  return hourParseTime(key) || dayParseTime(key) || key;
};

var normalizeData = function normalizeData(originalSerie) {
  var lookup = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var stackLookup = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var max = 0;
  var total = 0;
  var serie = (0, _lodash.transform)(originalSerie, function (liveObject, step, key) {
    var formattedKey = formatKey(key);

    if ((0, _lodash.isObject)(step)) {
      if (step.hasOwnProperty('c')) {
        liveObject[formattedKey] = {
          c: step.c,
          name: lookup[key] || formattedKey,
          key: key
        };
      } else {
        liveObject[formattedKey] = {
          c: 0,
          key: key,
          name: lookup[key] || formattedKey,
          stack: {}
        };
        (0, _lodash.each)(step, function (substep, subkey) {
          var formattedSubkey = formatKey(subkey);
          liveObject[formattedKey].c += substep.c;
          liveObject[formattedKey].stack[formattedSubkey] = {
            c: substep.c,
            name: stackLookup[subkey] || formattedSubkey,
            key: subkey
          };
        });
      }

      total += liveObject[formattedKey].c;
      max = liveObject[formattedKey].c > max ? liveObject[formattedKey].c : max;
    } else {
      total += step;
      max = step > max ? step : max;
      liveObject[formattedKey] = {
        c: step,
        key: key,
        name: lookup[key] || formattedKey,
        stack: {}
      };
    }

    return liveObject;
  }, {});
  return {
    serie: serie,
    total: total,
    max: max
  };
};

exports.normalizeData = normalizeData;
var numberFormat = (0, _d.format)(',d');
exports.numberFormat = numberFormat;

var getCommonPrefix = function getCommonPrefix(array) {
  // return empty string for arrays withe less then two elements
  if (!array || array.length <= 1) return ''; // Sorting an array and comparing the first and last elements

  var A = array.concat().sort(),
      a1 = A[0],
      a2 = A[A.length - 1],
      L = a1.length,
      i = 0;

  while (i < L && a1.charAt(i) === a2.charAt(i)) {
    i++;
  }

  var prefix = a1.substring(0, i); // Now checking if it contains number and ends with _

  if (prefix[prefix.length - 1] === '_' && !/\d/.test(prefix)) return prefix;else return '';
};

exports.getCommonPrefix = getCommonPrefix;